@@ -1,12 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # minimal sanity check
-from __future__ import print_function
 
 import sys
-import os
 
-from PIL import Image
-from PIL import features
+from PIL import Image, features
 
 try:
     Image.core.ping
@@ -17,17 +14,13 @@ except AttributeError:
     pass
 
 
-def _info(im):
-    im.load()
-    return im.format, im.mode, im.size
-
-
 def testimage():
     """
     PIL lets you create in-memory images with various pixel types:
 
     >>> from PIL import Image, ImageDraw, ImageFilter, ImageMath
     >>> im = Image.new("1", (128, 128)) # monochrome
+    >>> def _info(im): return (im.format, im.mode, im.size)
     >>> _info(im)
     (None, '1', (128, 128))
     >>> _info(Image.new("L", (128, 128))) # grayscale (luminance)
@@ -43,14 +36,14 @@ def testimage():
 
     Or open existing files:
 
-    >>> im = Image.open("Tests/images/hopper.gif")
-    >>> _info(im)
+    >>> with Image.open("Tests/images/hopper.gif") as im:
+    ...     _info(im)
     ('GIF', 'P', (128, 128))
     >>> _info(Image.open("Tests/images/hopper.ppm"))
     ('PPM', 'RGB', (128, 128))
     >>> try:
     ...  _info(Image.open("Tests/images/hopper.jpg"))
-    ... except IOError as v:
+    ... except OSError as v:
     ...  print(v)
     ('JPEG', 'RGB', (128, 128))
 
@@ -104,9 +97,9 @@ def testimage():
     10456
     >>> len(im.tobytes())
     49152
-    >>> _info(im.transform((512, 512), Image.AFFINE, (1,0,0,0,1,0)))
+    >>> _info(im.transform((512, 512), Image.Transform.AFFINE, (1,0,0,0,1,0)))
     (None, 'RGB', (512, 512))
-    >>> _info(im.transform((512, 512), Image.EXTENT, (32,32,96,96)))
+    >>> _info(im.transform((512, 512), Image.Transform.EXTENT, (32,32,96,96)))
     (None, 'RGB', (512, 512))
 
     The ImageDraw module lets you draw stuff in raster images:
@@ -150,9 +143,7 @@ def testimage():
     ('F', (128, 128))
 
     PIL can do many other things, but I'll leave that for another
-    day.  If you're curious, check the handbook, available from:
-
-        http://www.pythonware.com
+    day.
 
     Cheers /F
     """
@@ -163,32 +154,7 @@ if __name__ == "__main__":
 
     exit_status = 0
 
-    print("-" * 68)
-    print("Pillow", Image.__version__, "TEST SUMMARY ")
-    print("-" * 68)
-    print("Python modules loaded from", os.path.dirname(Image.__file__))
-    print("Binary modules loaded from", os.path.dirname(Image.core.__file__))
-    print("-" * 68)
-    for name, feature in [
-        ("pil", "PIL CORE"),
-        ("tkinter", "TKINTER"),
-        ("freetype2", "FREETYPE2"),
-        ("littlecms2", "LITTLECMS2"),
-        ("webp", "WEBP"),
-        ("transp_webp", "WEBP Transparency"),
-        ("webp_mux", "WEBPMUX"),
-        ("webp_anim", "WEBP Animation"),
-        ("jpg", "JPEG"),
-        ("jpg_2000", "OPENJPEG (JPEG2000)"),
-        ("zlib", "ZLIB (PNG/ZIP)"),
-        ("libtiff", "LIBTIFF"),
-        ("raqm", "RAQM (Bidirectional Text)"),
-    ]:
-        if features.check(name):
-            print("---", feature, "support ok")
-        else:
-            print("***", feature, "support not installed")
-    print("-" * 68)
+    features.pilinfo(sys.stdout, False)
 
     # use doctest to make sure the test program behaves as documented!
     import doctest
